from dataclasses import dataclass, field
from typing import List, Any, Union
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.dossier_parametrage.recupNomenclaturesRHResponse import NomenclatureList
import zeep
import logging

logger = logging.getLogger(__name__)


@dataclass
class RecupNomenclaturesRH(Zeepclient):
    """ L’objectif de ce flux est d’exposer tout ou une partie de la nomenclature RH d’un établissement
    donné aux applications externes.
    """
    code_repertoire: str = field(default='')                    # 3 caractère CDSTCO ZD00 (exemple: HJB = grade)
    code_administration: str = field(default='')    # 3 caractère (exemple: 'RUN' pour Université Réunion
    date_observation: str = field(default='')                   # format dd/mm/YYYY
    code_nomenclatures: str = field(default='')                 # 3 caractère CDCODE ZD00

    liste_nomenclatures: Any = field(init=False, default=None)

    def __post_init__(self):
        super().__post_init__()
        if self.code_repertoire is None and self.date_observation is None and self.code_nomenclatures is None:
            raise TypeError("Valeur requise pour au moins un champ.")
        self.__initialisation_object_liste()

    def response(self) -> Union[NomenclatureList, List]:
        """ Récuperer les informations sur tous les répertoires du référentiel SIHAM

        Returns:
            Liste contenant les informations concernant les valeurs présentes dans le répertoire
            - Code de l’administration
            - Code du répertoire de la nomenclature
            - Code (valeur)
            - Date de début de validité
            - Date de fin de validité
            - Libellé long
            - Libellé court
            - Type de rémunération, paramétrage du PIP
            - Témoin Enseignant/ chercheur BIATSS, paramétrage Groupe Hiérarchique
            - Témoin Enseignant/ chercheur BIATSS, paramétrage PIP


        """
        result = []
        try:
            resp = \
                self.zeep_client.service.RecupNomenclaturesRH(ParamNomenclature=
                                                              {'dateObservation': self.date_observation,
                                                               'codeAdministration': self.code_administration,
                                                               'listeNomenclatures': self.liste_nomenclatures})
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = NomenclatureList(nomenclatures=tmp_data)
                logger.debug(result)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def __initialisation_object_liste(self):
        """ Paramètre pour l'utilisation du webservice RecupNomenclaturesRH

        Returns:
            Un dictionnaire contenant les informations (paramètres) nécessaire pour l'utilisation
            du webservice RecupNomenclaturesRH

        """
        t_lnomenclatures = self.zeep_client.get_type('ns0:listeNomenclatures')

        self.liste_nomenclatures = t_lnomenclatures(codeNomenclature=self.code_nomenclatures,
                                                    codeRepertoire=self.code_repertoire)


