from datetime import datetime
from dataclasses import dataclass, field
from typing import List, Union
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.liste_agents.genericAgentsReponse import AgentDataModifieAffectList
from collections import OrderedDict

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupAgentsModifParAffect(Zeepclient):
    """ Le flux doit extraire tous les agents dont la position d’activité sont soit émérites dans l’établissement.
    """

    code_uo_affectation: str

    code_etablissement: str = field(default='')                 # code de l'établissement (exemple : 9740478B)
    date_debut_observation: str = field(default='')             # format YYYY-MM-DD-HH.MM.SS
    date_fin_observation: str = field(default='')
    found: tuple[bool, int] = \
        field(init=False, default=(False, -1), repr=False)  # tuple déterminant si un élément a été trouvé et son nombre

    def response(self) -> Union[AgentDataModifieAffectList, List]:
        """ Retourne les informations des agents dont l’un des attributs ci-dessous a été modifié dans l’intervalle de
        dates d’observation (date de saisie de la mise à jour de l’information HR Access
        située entre DateDebutObservation et DateFinObservation) :
            o	civilité,
            o	nom usuel,
            o	nom patronymique,
            o	prénom,
            o	date de naissance,
            o	affectation,
            o	entrée/sortie.


        Returns:
            Liste contenant les informations d'agents suivantes :
                - Matricule de l’agent
                - Qualité
                - Libellé de la qualité
                - Nom usuel
                - Prénom
                - Nom patronymique
                - Date de naissance
                - Unité organisationnelle
                - Libellé long de l’unité organisationnelle
                - Date d’effet de l’affectation
                - Date de fin de l’affectation
                - Catégorie statutaire
                - Libellé long de la catégorie statutaire
                - Pourcentage d’affectation
                -   Date de sortie administrative
                - Motif de sortie
                - Profil individuel paie
                - Libellé long du profil individuel paie
                - Code de l’emploi de l’affectation
                - Libellé long de l’emploi de l’affectation
                - Code poste de l’affectation dans l’établissement
                - Libellé long du code poste de l’affectation dans l’établissement


        """
        result = []
        try:
            resp = \
                self.zeep_client.service.RecupAgentsModifParAffect(ParamAgentModifie=
                                                                   {'codeEtablissement': self.code_etablissement,
                                                                    'codeUOAffectation': self.code_uo_affectation,
                                                                    'dateDebutObservation': self.ddo_datetime(),
                                                                    'dateFinObservation': self.dfo_datetime()})
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = AgentDataModifieAffectList(agent_data_modifie_affect=tmp_data)
                self.found = (True, len([ele for ele in result if isinstance(ele, dict)]))
            else:
                self.found = (False, 0)
                logger.debug('Aucun élément trouvé')
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def ddo_datetime(self) -> Union[datetime, str]:
        if self.date_debut_observation:
            return datetime.strptime(self.date_debut_observation, '%d/%m/%Y')
        return self.date_debut_observation

    def dfo_datetime(self) -> Union[datetime, str]:
        if self.date_fin_observation:
            return datetime.strptime(self.date_fin_observation, '%d/%m/%Y')
        return self.date_fin_observation
