from dataclasses import dataclass, field
from typing import Dict, Union
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.pec.genericPecResponse import GenericPecResponse

import zeep
import logging
import datetime

logger = logging.getLogger(__name__)


@dataclass
class CloreLeDossier(Zeepclient):
    """L’objectif de ce flux est de clore le dossier d'un agent.
    """
    categorie_situation: str
    date_sortie: datetime.datetime
    matricule: str
    motif_situation: str
    renouvellement_envisageable: str = field(default='')
    temoin_validite: str = field(default='')

    def response(self) -> Union[GenericPecResponse, Dict]:
        """L’objectif de ce web-service est de permettre la clôture du dossier de l’agent.
        L’appel à ce Web Service se fait par interrogation sur une liste de paramètres
        (matricule, date de sortie, catégories de sortie, etc.).
        Lors de l’appel, le Web Service vérifie qu’au moins une prise en charge a été réalisée pour cet agent et contrôle que la date d’entrée de l’agent (ZYES – DATENT) actuelle est antérieure à la date de sortie demandée.

        Returns:
            Le tableau de résultat de l’annulation contient les informations suivantes:
                o	Matricule de l’agent,
                o	Liste des anomalies rencontrées
                o	Statut de l’opération (Qui indique si l’annulation a bien été effectuée).
        """
        result = {}
        try:
            resp = self.zeep_client.service.CloreLeDossier(
                ParamCLD={'categorieSituation': self.categorie_situation,
                          'dateSortie': self.date_sortie,
                          'matricule': self.matricule,
                          'motifSituation': self.motif_situation,
                          'renouvellementEnvisageable': self.renouvellement_envisageable,
                          'temoinValidite': self.temoin_validite
                }
            )
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = GenericPecResponse(**tmp_data)
                logger.debug(result)
        except zeep.exceptions.Fault as fault:

            logger.error(fault)
        return result
