from dataclasses import dataclass, field
from typing import Dict, Union, Any
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.pec.genericPecResponse import GenericPecResponse

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RenouvellementAgent(Zeepclient):
    """L’objectif de ce flux est de renouveler des agents."""
    matricule: str
    categorie_entree: str           # 'ACTIVE'
    code_administration: str        # 'RUN'
    code_etablissement: str         # '9740478B'
    date_renouvellement: str      # format YYYY-MM-DD-HH.MM.SS
    emploi: str = field(default='')

    liste_carriere: Any = field(init=False, default=None)
    chevron_carriere: str = field(default='')
    date_effet_carriere: str = field(default='')       # format YYYY-MM-DD-HH.MM.SS
    date_fin: str = field(default='')                    # format YYYY-MM-DD-HH.MM.SS
    echelon: str = field(default='')
    grade: str = field(default='')
    indice_brut: str = field(default='')
    indice_majore: str = field(default='')
    mode_acces_echelon: str = field(default='')
    mode_acces_grade: str = field(default='')
    numCarriere: str = field(default='')
    numord: str = field(default='')
    qualite_statutaire: str = field(default='')
    tem_rem_acessoire: str = field(default='')
    temoin_validite: str = field(default='')

    liste_contrats: Any = field(init=False, default=None)
    chevron_contrat: str = field(default='')
    commentaire_contrat: str = field(default='')
    date_debut_contrat: str = field(default='')                    # format YYYY-MM-DD-HH.MM.SS
    date_fin_contrat: str = field(default='')                    # format YYYY-MM-DD-HH.MM.SS
    debut_periode_essai: str = field(default='')                    # format YYYY-MM-DD-HH.MM.SS
    devise_remuneration_contrat: str = field(default='')
    fin_periode_essai: str = field(default='')
    grade_tg: str = field(default='')
    hors_echelle_lettre: str = field(default='')
    indice_remu: str = field(default='')
    mode_de_gestion: str = field(default='')
    mode_remuneration: str = field(default='')
    nature_contrat: str = field(default='')
    nb_heures_contrat: str = field(default='')
    pourcentage: int = field(default=0)
    remuneration_contrat: int = field(default=0)
    taux_horaires: float = field(default=0)
    # temoin_validite
    type_contrat: str = field(default='')
    type_lien_juridique: str = field(default='')

    liste_modalites_services: Any = field(init=False, default=None)
    date_effet_modalite: str = field(default='')
    modalite: str = field(default='')
    # temoin_validite

    liste_positions: Any = field(init=False, default=None)
    date_effet_position: str = field(default='')
    date_fin_prevue: str = field(default='')
    date_fin_reelle: str = field(default='')
    position: str = field(default='')
    # temoin_validite

    # Important
    liste_statuts: Any = field(init=False, default=None)
    date_effet_statut: str = field(default='')
    statut: str = field(default='')
    # temoin_validite

    motif_entree: str = field(default='')
    poste: str = field(default='')
    # temoin_validite
    uo: str = field(default='')

    def __post_init__(self):
        super().__post_init__()
        self.__initialisation_object_liste()

    def response(self) -> Union[GenericPecResponse, Dict]:
        """ L’appel à ce Web Service se fait par interrogation sur une liste de paramètres
        (état civil, date de naissance, affectations, carrière, statuts, position etc.).
        Afin de tracer dans le dossier agent l’information indiquant que le dossier ait été renouvelé via web-service,
        la méthode lors de la création crée une occurrence de Matricules précédents (ZYU5) avec les valeurs suivantes :
            -	IDEMPR (matricule précédent) : Valorisé par ‘WS_REN’
            -	DTCHNG (date de changement) : Valorisé par la date du jour

        Ce flux doit retourner une table :
        -	Tableau de résultat du renouvellement
        (statut du renouvellement, matricule de l’agent, anomalie rencontrée) : une occurrence par agent.

        Returns:
            Tableau de résultat du renouvellement
            (statut du renouvellement, matricule de l’agent, anomalie rencontrée) : une occurrence par agent.
        """
        result = {}
        try:
            resp = self.zeep_client.service.RenouvellementAgent(
                ParamRA={'categorieEntree': self.categorie_entree,
                         'codeAdministration': self.code_administration,
                         'codeEtablissement': self.code_etablissement,
                         'dateRenouvellement': self.date_renouvellement,
                         'emploi': self.emploi,
                         'listeCarriere': self.liste_carriere,
                         'listeContrats': self.liste_contrats,
                         'listeModalitesServices': self.liste_modalites_services,
                         'listePositions': self.liste_positions,
                         'listeStatuts': self.liste_statuts,
                         'matricule': self.matricule,
                         'motifEntree': self.motif_entree,
                         'poste': self.poste, 'temoinValidite': self.temoin_validite, 'UO': self.uo
                         }
            )
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = GenericPecResponse(**tmp_data)
                logger.debug(result)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def __initialisation_object_liste(self):
        """ Paramètre pour l'utilisation du webservice PriseEnChargeAgent

        Returns:
            Un dictionnaire contenant les informations (paramètres) nécessaire pour l'utilisation
            du webservice PriseEnChargeAgent

        """
        t_lcarriere = self.zeep_client.get_type('ns0:listeCarriere')
        t_lcontrats = self.zeep_client.get_type('ns0:listeContrats')
        t_lmodalitesservice = self.zeep_client.get_type('ns0:listeModalitesServices')
        t_lpostions = self.zeep_client.get_type('ns0:listePositions')
        t_lstatus = self.zeep_client.get_type('ns0:listeStatuts')

        self.liste_carriere = t_lcarriere(chevron=self.chevron_carriere,
                                          dateEffetCarriere= self.date_effet_carriere,
                                          dateFin=self.date_fin,
                                          echelon=self.echelon, grade=self.grade, indiceBrut=self.indice_brut,
                                          indiceMajore=self.indice_majore, modeAccesEchelon=self.mode_acces_echelon,
                                          modeAccesGrade=self.mode_acces_grade, numCarriere=self.numCarriere,
                                          numord=self.numord, qualiteStatutaire=self.qualite_statutaire,
                                          temRemAcessoire=self.tem_rem_acessoire, temoinValidite=self.temoin_validite)

        self.liste_contrats = t_lcontrats(chevron=self.chevron_contrat, commentaireContrat=self.commentaire_contrat,
                                          dateDebutContrat=self.date_debut_contrat,
                                          dateFinContrat=self.date_fin_contrat,
                                          debutPeriodeEssai=self.debut_periode_essai,
                                          deviseRemunerationContrat=self.devise_remuneration_contrat,
                                          finPeriodeEssai=self.fin_periode_essai,
                                          gradeTG=self.grade_tg, horsEchelleLettre=self.hors_echelle_lettre,
                                          indiceRemu=self.indice_remu, modeDeGestion=self.mode_de_gestion,
                                          modeRemuneration=self.mode_remuneration, natureContrat=self.nature_contrat,
                                          nbHeuresContrat=self.nb_heures_contrat, pourcentage=self.pourcentage,
                                          remunerationContrat=self.remuneration_contrat,
                                          tauxHoraires=self.taux_horaires, temoinValidite=self.temoin_validite,
                                          typeContrat=self.type_contrat, typeLienJuridique=self.type_lien_juridique)

        self.liste_modalites_services = \
            t_lmodalitesservice(dateEffetModalite=self.date_effet_modalite,
                                modalite=self.modalite, temoinValidite=self.temoin_validite)

        self.liste_positions = t_lpostions(dateEffetPosition=self.date_effet_position,
                                           dateFinPrevue=self.date_fin_prevue,
                                           dateFinReelle=self.date_fin_reelle,
                                           position=self.position, temoinValidite=self.temoin_validite)

        self.liste_statuts = t_lstatus(dateEffetStatut=self.date_effet_statut,
                                       statut=self.statut, temoinValidite=self.temoin_validite)

