from dataclasses import dataclass
from configparser import ConfigParser
from wssiham.tools.toolbox import Toolbox
from typing import Dict
import pathlib


@dataclass
class Configuration:

    @classmethod
    def createws(cls, config_file_path: str, environment_name: str, user: str, password: str, base_wsdl: str,
                 path_certificat: str, default_wsdl: bool) -> str:
        """ Création fichier configuration des webservices SIHAM

        Args:
            config_file_path (str): Chemin du fichier de configuration des webservices SIHAM
            environment_name (str): Nom d'environnement (label) des webservices (exemple: prod, preprod, test, ...)
            user (str): Nom utilisateur webservice SIHAM
            password (str): Mot de passe webservice SIHAM
            base_wsdl (str): Base url du webservice SIHAM
            path_certificat (str): Chemin du certificat (crt)
            default_wsdl (str): Si oui, crée les endpoints par défaut des webservices SIHAM
                             Si non, aucun endpoint n'est créé

        Returns:
            Chemin absolu du fichier de configuration créé
        """
        file = f'{pathlib.Path(__file__).parents[1]}/{config_file_path}.ini'
        pwd_bytes = bytes(password, 'utf-8')
        config = ConfigParser()
        config.optionxform = str

        config.add_section('WS-SIHAM')
        config['WS-SIHAM']['ENVIRONMENT_NAME'] = environment_name
        config['WS-SIHAM']['USER'] = user
        config['WS-SIHAM']['PASSWORD'] = Toolbox.obscure(pwd_bytes).decode('utf-8')
        config['WS-SIHAM']['BASE_WSDL'] = base_wsdl
        config['WS-SIHAM']['CERTIFICAT'] = path_certificat

        if default_wsdl:
            config.add_section('DEFAULT-WSDL')
            config['DEFAULT-WSDL']['SIHAM_WS_LDAP'] = "siham-ws-ldap/Utilisateurs?wsdl"
            config['DEFAULT-WSDL']['DOSSIER_AGENT'] = "DossierAgentWebService/DossierAgentWebService?wsdl"
            config['DEFAULT-WSDL']['DOSSIER_AGENT_DATE'] = "DossierAgentDateWebService/DossierAgentDateWebService?wsdl"
            config['DEFAULT-WSDL']['DOSSIER_PARAMETRAGE'] \
                = "DossierParametrageWebService/DossierParametrageWebService?wsdl"
            config['DEFAULT-WSDL']['INDICATEUR'] = "IndicateurWebService/IndicateurWebService?wsdl"
            config['DEFAULT-WSDL']['LISTE_AGENTS'] = "ListeAgentsWebService/ListeAgentsWebService?wsdl"
            config['DEFAULT-WSDL']['LISTE_PARAMETRAGE'] = "ListeParametrageWebService/ListeParametrageWebService?wsdl"
            config['DEFAULT-WSDL']['NOMENCLATURE_PEC'] = "NomenclaturePECWebService/NomenclaturePECWebService?wsdl"
            config['DEFAULT-WSDL']['RECHERCHE_AGENT'] = "RechercheAgentWebService/RechercheAgentWebService?wsdl"
            config['DEFAULT-WSDL']['PEC'] = "PECWebService/PECWebService?wsdl"

        with open(file, 'w') as configfile:
            config.write(configfile)

        return file

    @staticmethod
    def loadws(file: str = Toolbox.abs_pathconf_ws()) -> Dict:
        """ Récupère les informations disponibles dans le ficiher de configuration des webservices SIHAM

        Returns:
            Retourne, sous forme de dictionnaire, les informations contenues dans le fichier de configuration
            des webservices SIHAM
        """

        configfile = file
        config = ConfigParser()
        config.read(configfile, encoding='utf-8')

        data = {'ENVIRONMENT_NAME': config['WS-SIHAM']['ENVIRONMENT_NAME'],
                'USER': config['WS-SIHAM']['USER'],
                'PASSWORD': config['WS-SIHAM']['PASSWORD'],
                'BASE_WSDL': config["WS-SIHAM"]["BASE_WSDL"],
                'CERTIFICAT': config["WS-SIHAM"]["CERTIFICAT"],
                'SIHAM_WS_LDAP': config["DEFAULT-WSDL"]["SIHAM_WS_LDAP"],
                'DOSSIER_AGENT_DATE': config["DEFAULT-WSDL"]["DOSSIER_AGENT_DATE"],
                'DOSSIER_AGENT': config["DEFAULT-WSDL"]["DOSSIER_AGENT"],
                'DOSSIER_PARAMETRAGE': config["DEFAULT-WSDL"]["DOSSIER_PARAMETRAGE"],
                'INDICATEUR': config["DEFAULT-WSDL"]["INDICATEUR"],
                'LISTE_AGENTS': config["DEFAULT-WSDL"]["LISTE_AGENTS"],
                'LISTE_PARAMETRAGE': config["DEFAULT-WSDL"]["LISTE_PARAMETRAGE"],
                'NOMENCLATURE_PEC': config["DEFAULT-WSDL"]["NOMENCLATURE_PEC"],
                'RECHERCHE_AGENT': config["DEFAULT-WSDL"]["RECHERCHE_AGENT"],
                'PEC': config["DEFAULT-WSDL"]["PEC"]}

        return data

    @staticmethod
    def create_confmod(environment: str, ws_config_file: str, dist_path: str = 'dist', docs_path: str = 'docs',
                       debug: bool = False, file: str = f"{pathlib.Path(__file__).parents[1]}/config.ini"):
        """ Crée le fichier de base de la configuration du module wssiham

        Args:
            environment (str): nom de l'environnement
            ws_config_file (str): chemin du fichier de configuration WebService SIHAM associé
            dist_path (str): chemin du dossier DIST (distribution). Par défaut : dist
            docs_path (str): chemin du dossier DOC (documentation). Par défaut : docs
            debug (bool): flag debuggage. Par défaut : False
            file (str): chemin du fichier de configration du module wssiham. Par défaut : config.ini

        """

        config = ConfigParser()
        config.optionxform = str

        config.add_section('APP')
        config['APP']['ENVIRONMENT'] = environment
        config['APP']['DEBUG'] = str(debug)
        config['APP']['WS_CONFIG_FILE'] = ws_config_file
        config['APP']['DIST_PATH'] = dist_path
        config['APP']['DOCS_PATH'] = docs_path

        with open(file, 'w+') as configfile:
            config.write(configfile)


